package edu.jhmi.telometer.bean;

import edu.jhmi.telometer.controller.ScoreController;
import edu.jhmi.telometer.util.FrameUtil;
import edu.jhmi.telometer.util.NucleusHolder;
import edu.jhmi.telometer.util.OverlayImageCanvas;
import ij.ImagePlus;
import ij.gui.ImageCanvas;
import ij.gui.ImageWindow;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

@Data
@Slf4j
public class Model {

    private ImageMakeup cy3ImageMakeup;
    private ImageMakeup dapiImageMakeup;
    private ImageMakeup maskedCy3ImageMakeup;
    private ImageMakeup combinedImageMakeup;
    private ImageMakeup annotatedImageMakeup;
    private NucleusHolder nucleusHolder = new NucleusHolder();

    public ImagePlus getCy3ImagePlus() {
        return cy3ImageMakeup.getImagePlus();
    }

    public ImagePlus getDapiImagePlus() {
        return dapiImageMakeup.getImagePlus();
    }

    public ImagePlus getMaskedCy3ImagePlus() {
        return maskedCy3ImageMakeup.getImagePlus();
    }


    public void clearImageMakeup(ImageType imageType) {
        Model model = this;
        switch (imageType){
            case CY3:
                model.setCy3ImageMakeup(null);
                break;
            case DAPI:
                model.setDapiImageMakeup(null);
                break;
            case MASKED_CY3:
                model.setMaskedCy3ImageMakeup(null);
                break;
            case COMBINED:
                model.setCombinedImageMakeup(null);
                break;
            case ANNOTATED:
                model.setAnnotatedImageMakeup(null);
                break;
            default:
                throw new RuntimeException("bad imageType: " + imageType);
        }
    }

    public ImageMakeup getImageMakeup(ImageType imageType) {
        Model model = this;
        switch (imageType) {
            case CY3:
                return model.getCy3ImageMakeup();
            case DAPI:
                return model.getDapiImageMakeup();
            case MASKED_CY3:
                return model.getMaskedCy3ImageMakeup();
            case COMBINED:
                return model.getCombinedImageMakeup();
            case ANNOTATED:
                return model.getAnnotatedImageMakeup();
            default:
                throw new RuntimeException("bad imageType: " + imageType);
        }
    }

    public ImagePlus getImagePlus(ImageType imageType) {
        Model model = this;
        ImageMakeup imageMakeup = model.getImageMakeup(imageType);
        if (imageMakeup == null) {
            return null;
        }
        return imageMakeup.getImagePlus();
    }


    public void initModelImagePlus(ImagePlus imagePlus, ImageType imageType) {
        log.info("initModelImagePlus. ImageType:{}", imageType);
        ImageMakeup imageMakeup = imagePlusToImageMakeup(imagePlus, imageType);
        switch (imageType) {
            case CY3:
                setCy3ImageMakeup(imageMakeup);
                break;
            case DAPI:
                setDapiImageMakeup(imageMakeup);
                break;
            case MASKED_CY3:
                setMaskedCy3ImageMakeup(imageMakeup);
                break;
            case COMBINED:
                setCombinedImageMakeup(imageMakeup);
                break;
            case ANNOTATED:
                setAnnotatedImageMakeup(imageMakeup);
                break;
            default:
                throw new RuntimeException("bad imageType: " + this);
        }
    }

    private Set<ImageType> imageTypesUsingOverlay = new HashSet(
            Arrays.asList(ImageType.DAPI, ImageType.CY3));

    private ImageMakeup imagePlusToImageMakeup(ImagePlus imagePlus, ImageType imageType) {
        ImageCanvas imageCanvas = imageTypesUsingOverlay.contains(imageType) ? new OverlayImageCanvas(imagePlus, nucleusHolder)
                : new ImageCanvas(imagePlus);
        ImageWindow imageWindow = new ImageWindow(imagePlus, imageCanvas);
        FrameUtil.setImageWindowLocation(imageWindow);
        return ImageMakeup.builder()
                .imagePlus(imagePlus)
                .imageWindow(imageWindow)
                .imageCanvas(imageCanvas)
                .build();
    }

}
