package edu.jhmi.telometer.bean;

import lombok.Builder;
import lombok.Data;
import lombok.Singular;
import lombok.experimental.Tolerate;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

@Data
@Builder
public class Particle {

    @Singular
    private Set<Point> points;

    @Tolerate
    public Particle() {
        points = new HashSet();
    }


    public static Particle createFromSinglePoint(Point point) {
        return createFromParticlesAndPoint(Collections.emptySet(), point);
    }

    public static Particle createFromParticleAndPoint(Particle particle, Point point) {
        return createFromParticlesAndPoint(Collections.singleton(particle), point);
    }

    public static Particle createFromParticlesAndPoint(Set<Particle> particles, Point point) {
        Particle particle = new Particle();
        for (Particle p : particles) {
            particle.points.addAll(p.getPoints());
        }
        particle.points.add(point);
        return particle;
    }


    public static Particle createFromParticles(Set<Particle> particles) {
        Particle uberParticle = new Particle();
        for (Particle particle : particles) {
            uberParticle.points.addAll(particle.getPoints());
        }
        return uberParticle;

    }

    public static Particle createFromParticles(Particle particle1, Particle particle2) {
        Particle particle = new Particle();
        particle.points.addAll(particle1.getPoints());
        particle.points.addAll(particle2.getPoints());
        return particle;
    }

    public void addPoint(Point point) {
        points.add(point);
    }
}

