package edu.jhmi.telometer.calc.impl;

import edu.jhmi.telometer.bean.Image;
import edu.jhmi.telometer.bean.Mask;
import edu.jhmi.telometer.bean.Pixelation;
import edu.jhmi.telometer.calc.api.PixelModifier;
import ij.ImagePlus;
import ij.gui.Roi;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

public class PixelModifierImpl implements PixelModifier {

    @Override
    public void applyRoiToImagePlus(Roi roi, ImagePlus imagePlus) {
        final Color DRAW_COLOR = Color.WHITE;
        try {
            imagePlus.lock();
            imagePlus.setColor(DRAW_COLOR);
            roi.drawPixels(imagePlus.getProcessor());
        } finally {
            imagePlus.unlock();
        }
    }

    @Override
    public Pixelation subtractIntensity(Pixelation pixelation, int intensity) {
        List<Integer> pixels = pixelation.getPixels();
        List<Integer> returned = new ArrayList();
        for (int pixel : pixels){
            pixel = subtractIntensity(pixel, intensity);
            returned.add(pixel);

        }
        return Pixelation.builder().pixels(returned).build();
    }

    private static int subtractIntensity(int pixel, int intensity) {
        int newPixel = pixel - intensity;
        if (newPixel < 0) {
            newPixel = 0;
        }
        return newPixel;
    }


    @Override
    public Image zeroOutNonMask(Image image, Mask mask) {

        List<Integer> newPixels = new ArrayList();
        List<Integer> pixels = image.getPixelation().getPixels();
        int size = mask.getBools().size();
        for (int i = 0; i < size; i++){
            boolean inMask = mask.isPositionInMask(i);
            int value = inMask ? pixels.get(i) : 0;
            newPixels.add(value);
        }
        return Image.builder().dimension(image.getDimension()).pixelation(
                Pixelation.builder().pixels(newPixels).build()).build();

    }
}
