package edu.jhmi.telometer.thirdparty;

//a permutation of the standard ImageJ plugin which does a few things for this project

import edu.jhmi.telometer.view.admin.PresetDialog;
import ij.IJ;
import ij.ImageJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.GenericDialog;
import ij.gui.TrimmedButton;
import ij.measure.Calibration;
import ij.measure.Measurements;
import ij.plugin.frame.PasteController;
import ij.plugin.frame.Recorder;
import ij.process.ByteProcessor;
import ij.process.FloatProcessor;
import ij.process.ImageProcessor;
import ij.process.ImageStatistics;
import ij.process.ShortProcessor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import javax.swing.JFrame;
import javax.swing.JPanel;
import java.awt.Button;
import java.awt.Canvas;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Rectangle;
import java.awt.Scrollbar;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.ColorModel;
import java.awt.image.IndexColorModel;

/**
 * Adjusts the lower and upper threshold levels of the active image. This class is
 * multi-threaded to provide a more responsive user interface.
 */
public class Threshold_Adjuster_
        implements Measurements, Runnable, ActionListener, AdjustmentListener, ItemListener {

    static final int RED = 0, BLACK_AND_WHITE = 1, OVER_UNDER = 2;
    static final String[] modes = { "Red", "Black & White", "Over/Under" };
    static final double defaultMinThreshold = 10;
    static final double defaultMaxThreshold = 255;
    static boolean fill1 = true;
    static boolean fill2 = true;
    static boolean useBW = true;
    //static Frame instance;
    static int mode = RED;
    private static final Logger logger = LoggerFactory.getLogger(Threshold_Adjuster_.class);

    ThresholdPlot plot = new ThresholdPlot();
    Thread thread;

    int minValue = -1;
    int maxValue = -1;
    int sliderRange = 256;
    boolean doAutoAdjust, doReset, doApplyLut, doStateChange, doSet;

    Panel panel;
    Button autoB, resetB, applyB, setB;
    int previousImageID;
    int previousImageType;
    double previousMin, previousMax;
    ImageJ ij;
    double minThreshold, maxThreshold; // 0-255
    public Scrollbar minSlider, maxSlider; //TODO
    Label label1, label2;
    boolean done;
    boolean invertedLut;
    int lutColor;
    static Choice choice;
    private JPanel mainPanel = new JPanel();

    public Threshold_Adjuster_() {
        //super("Threshold");
//        if (instance != null) {
//            instance.toFront();
//            return;
//        }

        //WindowManager.addWindow(this);
        //instance = this;
        setLutColor(mode);
        IJ.register(PasteController.class);

        ij = IJ.getInstance();
        Font font = new Font("SansSerif", Font.PLAIN, 10);
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        setLayout(gridbag);

        // plot
        int y = 0;
        c.gridx = 0;
        c.gridy = y++;
        c.gridwidth = 2;
        c.fill = GridBagConstraints.BOTH;
        c.anchor = GridBagConstraints.CENTER;
        c.insets = new Insets(10, 10, 0, 10);
        add(plot, c);

        // minThreshold slider
        minSlider = new Scrollbar(Scrollbar.HORIZONTAL, 0, 1, 0, sliderRange);
        c.gridx = 0;
        c.gridy = y++;
        c.gridwidth = 1;
        c.weightx = IJ.isMacintosh() ? 90 : 100;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.insets = new Insets(5, 10, 0, 0);
        add(minSlider, c);
        minSlider.addAdjustmentListener(this);
        minSlider.setUnitIncrement(1);

        // minThreshold slider label
        c.gridx = 1;
        c.gridwidth = 1;
        c.weightx = IJ.isMacintosh() ? 10 : 0;
        c.insets = new Insets(5, 0, 0, 10);
        label1 = new Label("       ", Label.RIGHT);
        label1.setFont(font);
        add(label1, c);

        // maxThreshold slider
        maxSlider = new Scrollbar(Scrollbar.HORIZONTAL, sliderRange, 1, 0, sliderRange);
        c.gridx = 0;
        c.gridy = y++;
        c.gridwidth = 1;
        c.weightx = 100;
        c.insets = new Insets(0, 10, 0, 0);
        add(maxSlider, c);
        maxSlider.addAdjustmentListener(this);
        maxSlider.setUnitIncrement(1);

        // maxThreshold slider label
        c.gridx = 1;
        c.gridwidth = 1;
        c.weightx = 0;
        c.insets = new Insets(0, 0, 0, 10);
        label2 = new Label("       ", Label.RIGHT);
        label2.setFont(font);
        add(label2, c);

        // choice
        choice = new Choice();
        for (int i = 0; i < modes.length; i++)
            choice.addItem(modes[i]);
        choice.select(mode);
        choice.addItemListener(this);
        c.gridx = 0;
        c.gridy = y++;
        c.gridwidth = 2;
        c.insets = new Insets(5, 5, 0, 5);
        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.NONE;
        add(choice, c);

        // buttons
        int trim = IJ.isMacOSX() ? 15 : 0;
        panel = new Panel();
        autoB = new TrimmedButton("Auto", trim);
        autoB.addActionListener(this);
        autoB.addKeyListener(ij);
        panel.add(autoB);
        applyB = new TrimmedButton("Apply", trim);
        applyB.addActionListener(this);
        applyB.addKeyListener(ij);
        panel.add(applyB);
        resetB = new TrimmedButton("Reset", trim);
        resetB.addActionListener(this);
        resetB.addKeyListener(ij);
        panel.add(resetB);
        setB = new TrimmedButton("Set", trim);
        setB.addActionListener(this);
        setB.addKeyListener(ij);
        panel.add(setB);
        c.gridx = 0;
        c.gridy = y++;
        c.gridwidth = 2;
        c.insets = new Insets(0, 5, 10, 5);
        add(panel, c);

//        JFrame frame = new JFrame("Thresholder-test");
//        frame.getContentPane().add(mainPanel);
//        frame.pack();
        //UNCOMMENT TO DISPLAY FRAME   frame.setVisible(true);
        //addKeyListener(ij); // ImageJ handles keyboard shortcuts
        //mainPanel.pack();
        //GUI.center(this);
        //mainPanel.setVisible(true);

        thread = new Thread(this, "ThresholdAdjuster");
        // thread.setPriority(thread.getPriority()-1);

        thread.start();
        ImagePlus imp = WindowManager.getCurrentImage();
        //logger.debug("TA imp: {}", imp);
        if (imp != null)
            setup(imp);
    }

    private void add(Component panel, GridBagConstraints c) {
        mainPanel.add(panel, c);
    }

    private void setLayout(GridBagLayout gridbag) {
        mainPanel.setLayout(gridbag);
    }

    public synchronized void adjustmentValueChanged(AdjustmentEvent e) {
        if (e.getSource() == minSlider)
            minValue = minSlider.getValue();
        else
            maxValue = maxSlider.getValue();
        notify();
    }

    public synchronized void actionPerformed(ActionEvent e) {
        Button b = (Button) e.getSource();
        if (b == null)
            return;
        if (b == resetB)
            doReset = true;
        else if (b == autoB)
            doAutoAdjust = true;
        else if (b == applyB)
            doApplyLut = true;
        else if (b == setB)
            doSet = true;
        notify();
    }

    void setLutColor(int mode) {
        switch (mode) {
        case RED:
            lutColor = ImageProcessor.RED_LUT;
            break;
        case BLACK_AND_WHITE:
            lutColor = ImageProcessor.BLACK_AND_WHITE_LUT;
            break;
        case OVER_UNDER:
            lutColor = ImageProcessor.OVER_UNDER_LUT;
            break;
        }
    }

    public synchronized void itemStateChanged(ItemEvent e) {
        mode = choice.getSelectedIndex();
        setLutColor(mode);
        doStateChange = true;
        notify();
    }

    ImageProcessor setup(ImagePlus imp) {
        ImageProcessor ip;
        int type = imp.getType();
        if (type == ImagePlus.COLOR_RGB)
            return null;
        ip = imp.getProcessor();
        boolean minMaxChange = false;
        if (type == ImagePlus.GRAY16 || type == ImagePlus.GRAY32) {
            if (ip.getMin() != previousMin || ip.getMax() != previousMax)
                minMaxChange = true;
            previousMin = ip.getMin();
            previousMax = ip.getMax();
        }
        int id = imp.getID();
        if (minMaxChange || id != previousImageID || type != previousImageType) {
            invertedLut = imp.isInvertedLut();
            minThreshold = ip.getMinThreshold();
            maxThreshold = ip.getMaxThreshold();
            if (minThreshold == ImageProcessor.NO_THRESHOLD) {
                minThreshold = defaultMinThreshold;
                maxThreshold = defaultMaxThreshold;
            }
            plot.setHistogram(imp);
            scaleUpAndSet(ip, minThreshold, maxThreshold);
            updateLabels(imp, ip);
            updatePlot();
            updateScrollBars();
            imp.updateAndDraw();
        }
        previousImageID = id;
        previousImageType = type;
        return ip;
    }

    /** Scales threshold levels in the range 0-255 to the actual levels. */
    void scaleUpAndSet(ImageProcessor ip, double minThreshold, double maxThreshold) {
        if (!(ip instanceof ByteProcessor) && minThreshold != ImageProcessor.NO_THRESHOLD) {
            double min = ip.getMin();
            double max = ip.getMax();
            if (max > min) {
                minThreshold = min + (minThreshold / 255.0) * (max - min);
                maxThreshold = min + (maxThreshold / 255.0) * (max - min);
            } else
                minThreshold = ImageProcessor.NO_THRESHOLD;
        }
        ip.setThreshold(minThreshold, maxThreshold, lutColor);
    }

    /** Scales a threshold level to the range 0-255. */
    double scaleDown(ImageProcessor ip, double threshold) {
        double min = ip.getMin();
        double max = ip.getMax();
        if (max > min)
            return ((threshold - min) / (max - min)) * 255.0;
        else
            return ImageProcessor.NO_THRESHOLD;
    }

    /** Scales a threshold level in the range 0-255 to the actual level. */
    double scaleUp(ImageProcessor ip, double threshold) {
        double min = ip.getMin();
        double max = ip.getMax();
        if (max > min)
            return min + (threshold / 255.0) * (max - min);
        else
            return ImageProcessor.NO_THRESHOLD;
    }

    void updatePlot() {
        plot.minThreshold = minThreshold;
        plot.maxThreshold = maxThreshold;
        plot.mode = mode;
        plot.repaint();
    }

    void updateLabels(ImagePlus imp, ImageProcessor ip) {
        double min = ip.getMinThreshold();
        double max = ip.getMaxThreshold();
        if (min == ImageProcessor.NO_THRESHOLD) {
            label1.setText("");
            label2.setText("");
        } else {
            Calibration cal = imp.getCalibration();
            if (cal.calibrated()) {
                min = cal.getCValue((int) min);
                max = cal.getCValue((int) max);
            }
            if (((int) min == min && (int) max == max) || (ip instanceof ShortProcessor)) {
                label1.setText("" + (int) min);
                label2.setText("" + (int) max);
            } else {
                label1.setText("" + IJ.d2s(min, 2));
                label2.setText("" + IJ.d2s(max, 2));
            }
        }
    }

    void updateScrollBars() {
        minSlider.setValue((int) minThreshold);
        maxSlider.setValue((int) maxThreshold);
    }

    void adjustMinThreshold(ImagePlus imp, ImageProcessor ip, double value) {
        if (IJ.altKeyDown()) {
            double width = maxThreshold - minThreshold;
            if (width < 1.0)
                width = 1.0;
            minThreshold = value;
            maxThreshold = minThreshold + width;
            if ((minThreshold + width) > 255) {
                minThreshold = 255 - width;
                maxThreshold = minThreshold + width;
                minSlider.setValue((int) minThreshold);
            }
            maxSlider.setValue((int) maxThreshold);
            scaleUpAndSet(ip, minThreshold, maxThreshold);
            return;
        }
        minThreshold = value;
        if (maxThreshold < minThreshold) {
            maxThreshold = minThreshold;
            maxSlider.setValue((int) maxThreshold);
        }
        scaleUpAndSet(ip, minThreshold, maxThreshold);
    }

    void adjustMaxThreshold(ImagePlus imp, ImageProcessor ip, int cvalue) {
        maxThreshold = cvalue;
        if (minThreshold > maxThreshold) {
            minThreshold = maxThreshold;
            minSlider.setValue((int) minThreshold);
        }
        scaleUpAndSet(ip, minThreshold, maxThreshold);
    }

    void reset(ImagePlus imp, ImageProcessor ip) {
        plot.setHistogram(imp);
        ip.resetThreshold();
        updateScrollBars();
        if (Recorder.record)
            Recorder.record("resetThreshold");
    }

    void doSet(ImagePlus imp, ImageProcessor ip) {
        double level1 = ip.getMinThreshold();
        double level2 = ip.getMaxThreshold();
        if (level1 == ImageProcessor.NO_THRESHOLD) {
            level1 = scaleUp(ip, defaultMinThreshold);
            level2 = scaleUp(ip, defaultMaxThreshold);
        }
        Calibration cal = imp.getCalibration();
        int digits = (ip instanceof FloatProcessor) || cal.calibrated() ? 2 : 0;
        level1 = cal.getCValue(level1);
        level2 = cal.getCValue(level2);
        GenericDialog gd = new GenericDialog("Set Threshold Levels");
        gd.addNumericField("Lower Threshold Level: ", level1, digits);
        gd.addNumericField("Upper Threshold Level: ", level2, digits);
        gd.showDialog();
        if (gd.wasCanceled())
            return;
        level1 = gd.getNextNumber();
        level2 = gd.getNextNumber();
        level1 = cal.getRawValue(level1);
        level2 = cal.getRawValue(level2);
        if (level2 < level1)
            level2 = level1;
        double minDisplay = ip.getMin();
        double maxDisplay = ip.getMax();
        ip.resetMinAndMax();
        double minValue = ip.getMin();
        double maxValue = ip.getMax();
        if (level1 < minValue)
            level1 = minValue;
        if (level2 > maxValue)
            level2 = maxValue;
        boolean outOfRange = level1 < minDisplay || level2 > maxDisplay;
        if (outOfRange)
            plot.setHistogram(imp);
        else
            ip.setMinAndMax(minDisplay, maxDisplay);

        minThreshold = scaleDown(ip, level1);
        maxThreshold = scaleDown(ip, level2);
        scaleUpAndSet(ip, minThreshold, maxThreshold);
        updateScrollBars();
        if (Recorder.record)
            Recorder.record("setThreshold", (int) ip.getMinThreshold(), (int) ip.getMaxThreshold());
    }

    void changeState(ImagePlus imp, ImageProcessor ip) {
        scaleUpAndSet(ip, minThreshold, maxThreshold);
        updateScrollBars();
    }

    void autoThreshold(ImagePlus imp, ImageProcessor ip) {
        if (!(ip instanceof ByteProcessor))
            return;
        ImageStatistics stats = imp.getStatistics(MIN_MAX + MODE);
        int threshold = ((ByteProcessor) ip).getAutoThreshold();
        if ((stats.max - stats.mode) < (stats.mode - stats.min)) {
            minThreshold = stats.min;
            maxThreshold = threshold;
        } else {
            minThreshold = threshold;
            maxThreshold = stats.max;
        }
        scaleUpAndSet(ip, minThreshold, maxThreshold);
        updateScrollBars();
        if (Recorder.record)
            Recorder.record("setThreshold", (int) ip.getMinThreshold(), (int) ip.getMaxThreshold());
    }

    public void apply(double despeckleRatio) {
        try {
            //IJ.runPlugIn("edu.jhmi.telometer3.thirdparty.Thresholder_", "");
            Thresholder_ thresholder = new Thresholder_();
            thresholder.run(WindowManager.getCurrentImage().getProcessor());
            //TODO tc.threshold = (int) minThreshold;
            despeckle(WindowManager.getCurrentImage(), despeckleRatio);
            //windowClosing(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
            // this.setVisible(false);
            // this.dispose();
            // this.finalize();
        } catch (Throwable t) {
        	logger.error("General Error: " + t, t);
        }
    }

    static final int RESET = 0, AUTO = 1, HIST = 2, APPLY = 3, STATE_CHANGE = 4, MIN_THRESHOLD = 5, MAX_THRESHOLD = 6, SET = 7;

    // Separate thread that does the potentially time-consuming processing
    public void run() {
        while (!done) {
            synchronized (this) {
                try {
                    wait();
                } catch (InterruptedException e) {
                }
            }
            doUpdate();
        }
    }

    void doUpdate() {
        ImagePlus imp;
        ImageProcessor ip;
        int action;
        int min = minValue;
        int max = maxValue;
        if (doReset)
            action = RESET;
        else if (doAutoAdjust)
            action = AUTO;
        else if (doApplyLut)
            action = APPLY;
        else if (doStateChange)
            action = STATE_CHANGE;
        else if (doSet)
            action = SET;
        else if (minValue >= 0)
            action = MIN_THRESHOLD;
        else if (maxValue >= 0)
            action = MAX_THRESHOLD;
        else
            return;
        minValue = -1;
        maxValue = -1;
        doReset = false;
        doAutoAdjust = false;
        doApplyLut = false;
        doStateChange = false;
        doSet = false;
        imp = WindowManager.getCurrentImage();
        if (imp == null) {
            IJ.beep();
            IJ.showStatus("No image");
            return;
        }
        ip = setup(imp);
        if (ip == null) {
            imp.unlock();
            IJ.beep();
            IJ.showStatus("RGB images cannot be thresolded");
            return;
        }
        // IJ.write("setup: "+(imp==null?"null":imp.getTitle()));
        switch (action) {
        case RESET:
            reset(imp, ip);
            break;
        case AUTO:
            autoThreshold(imp, ip);
            break;
        case APPLY:
            //apply();
            throw new RuntimeException("not implemented because we don't have the despeckleRatio");
            //break;
        case STATE_CHANGE:
            changeState(imp, ip);
            break;
        case SET:
            doSet(imp, ip);
            break;
        case MIN_THRESHOLD:
            adjustMinThreshold(imp, ip, min);
            break;
        case MAX_THRESHOLD:
            adjustMaxThreshold(imp, ip, max);
            break;
        }
        updatePlot();
        updateLabels(imp, ip);
        ip.setLutAnimation(true);
        imp.updateAndDraw();
    }

//    public void windowClosing(WindowEvent e) {
//        super.windowClosing(e);
//        instance = null;
//        done = true;
//        synchronized (this) {
//            notify();
//        }
//    }


    void despeckle(ImagePlus imp, double despeckleRatio) {
        Assert.isTrue(!PresetDialog.isDespeckleRatioBad(despeckleRatio), PresetDialog.getDespeckleErrorMessage(despeckleRatio));
        ImageProcessor ip = imp.getProcessor();
        //logger.debug("ip: " + ip);

        int p1, p2, p3, p4, p5, p6, p7, p8, p9;
        byte[] pixels = (byte[]) imp.getProcessor().getPixels();
        byte[] pixels2 = new byte[pixels.length];
        System.arraycopy(pixels, 0, pixels2, 0, pixels.length);

        int offset;
        //int sum1, sum2 = 0, sum = 0;
        int[] values = new int[10];

        int yMin = 1;
        int yMax = imp.getProcessor().getHeight() - 2;
        int xMin = 1;
        int xMax = imp.getProcessor().getWidth() - 2;
        int width = imp.getProcessor().getWidth();
        int rowOffset = width;
        //int count = 0;
        for (int y = yMin; y <= yMax; y++) {
            offset = xMin + y * width;
            p2 = pixels2[offset - rowOffset - 1] & 0xff;
            p3 = pixels2[offset - rowOffset] & 0xff;
            p5 = pixels2[offset - 1] & 0xff;
            p6 = pixels2[offset] & 0xff;
            p8 = pixels2[offset + rowOffset - 1] & 0xff;
            p9 = pixels2[offset + rowOffset] & 0xff;

            for (int x = xMin; x <= xMax; x++) {
                p1 = p2;
                p2 = p3;
                p3 = pixels2[offset - rowOffset + 1] & 0xff;
                p4 = p5;
                p5 = p6;
                p6 = pixels2[offset + 1] & 0xff;
                p7 = p8;
                p8 = p9;
                p9 = pixels2[offset + rowOffset + 1] & 0xff;

                values[1] = p1;
                values[2] = p2;
                values[3] = p3;
                values[4] = p4;
                values[5] = p5;
                values[6] = p6;
                values[7] = p7;
                values[8] = p8;
                values[9] = p9;

                pixels[offset++] = (byte) findMedian(values, (float) despeckleRatio);
            }
        }
        imp.getProcessor().setPixels(pixels);
        imp.updateAndDraw();
    }

    private final int findMedian(int[] values, float ratio) {
        int whiteCount = 0;
        int blackCount = 0;
        for (int i = 0; i < values.length; i++) {
            if (values[i] == 0xff)
                whiteCount++;
            else
                blackCount++;
        }
        /*
         * if (blackCount <= 0) return 0xFF; else
         */
        if ((((float) whiteCount) / 8f) >= ratio)
            return 0xFF;
        else
            return 0x00;
    }


    public void setMinValue(int newValue) {
        minSlider.setValue(newValue);
        adjustmentValueChanged(new AdjustmentEvent(minSlider, 1, 1, newValue));
    }
    public int getMinValue(){
        return minSlider.getValue();
    }
} // ThresholdAdjuster class

class ThresholdPlot extends Canvas
        implements Measurements, MouseListener {

    static final int WIDTH = 256, HEIGHT = 48;
    double minThreshold = 85;
    double maxThreshold = 170;
    int[] histogram;
    Color[] hColors;
    int hmax;
    Image os;
    Graphics osg;
    int mode;

    public ThresholdPlot() {
        addMouseListener(this);
        setSize(WIDTH + 1, HEIGHT + 1);
    }

    void setHistogram(ImagePlus imp) {
        ImageProcessor ip = imp.getProcessor();
        if (!(ip instanceof ByteProcessor)) {
            double min = ip.getMin();
            double max = ip.getMax();
            ip.setMinAndMax(min, max);
            Rectangle r = ip.getRoi();
            ip = new ByteProcessor(ip.createImage());
            ip.setRoi(r);
        }
        ip.setMask(imp.getMask());
        ImageStatistics stats = ImageStatistics.getStatistics(ip, AREA + MODE, null);
        int maxCount2 = 0;
        histogram = stats.histogram;
        for (int i = 0; i < stats.nBins; i++)
            if ((histogram[i] > maxCount2) && (i != stats.mode))
                maxCount2 = histogram[i];
        hmax = stats.maxCount;
        if ((hmax > (maxCount2 * 2)) && (maxCount2 != 0)) {
            hmax = (int) (maxCount2 * 1.5);
            histogram[stats.mode] = hmax;
        }
        os = null;

        ColorModel cm = ip.getColorModel();
        if (!(cm instanceof IndexColorModel))
            return;
        IndexColorModel icm = (IndexColorModel) cm;
        int mapSize = icm.getMapSize();
        if (mapSize != 256)
            return;
        byte[] r = new byte[256];
        byte[] g = new byte[256];
        byte[] b = new byte[256];
        icm.getReds(r);
        icm.getGreens(g);
        icm.getBlues(b);
        hColors = new Color[256];
        for (int i = 0; i < 256; i++)
            hColors[i] = new Color(r[i] & 255, g[i] & 255, b[i] & 255);
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void paint(Graphics g) {
        if (histogram != null) {
            if (os == null) {
                os = createImage(WIDTH, HEIGHT);
                osg = os.getGraphics();
                osg.setColor(Color.white);
                osg.fillRect(0, 0, WIDTH, HEIGHT);
                osg.setColor(Color.gray);
                for (int i = 0; i < WIDTH; i++) {
                    if (hColors != null)
                        osg.setColor(hColors[i]);
                    osg.drawLine(i, HEIGHT, i, HEIGHT - ((int) (HEIGHT * histogram[i]) / hmax));
                }
                osg.dispose();
            }
            g.drawImage(os, 0, 0, this);
        } else {
            g.setColor(Color.white);
            g.fillRect(0, 0, WIDTH, HEIGHT);
        }
        g.setColor(Color.black);
        g.drawRect(0, 0, WIDTH, HEIGHT);
        if (mode == Threshold_Adjuster_.RED)
            g.setColor(Color.red);
        else if (mode == Threshold_Adjuster_.OVER_UNDER) {
            g.setColor(Color.blue);
            g.drawRect(1, 1, (int) minThreshold - 2, HEIGHT);
            g.drawRect(1, 0, (int) minThreshold - 2, 0);
            g.setColor(Color.green);
            g.drawRect((int) maxThreshold + 1, 1, WIDTH - (int) maxThreshold, HEIGHT);
            g.drawRect((int) maxThreshold + 1, 0, WIDTH - (int) maxThreshold, 0);
            return;
        }
        g.drawRect((int) minThreshold, 1, (int) (maxThreshold - minThreshold), HEIGHT);
        g.drawLine((int) minThreshold, 0, (int) maxThreshold, 0);
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }


} // ThresholdPlot class
