package edu.jhmi.telometer.util;

import edu.jhmi.telometer.bean.NucleusEntry;
import edu.jhmi.telometer.bean.Point;
import ij.ImagePlus;
import ij.gui.ImageCanvas;
import ij.gui.Roi;
import lombok.extern.slf4j.Slf4j;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Set;

/** Without altering the imagePlus, OverlayImageCanvas supports drawing ROIs over an imagePlus
 * (unlike RoiToImagePlusApplier)
 */
@Slf4j
public class OverlayImageCanvas extends ImageCanvas {

    private NucleusHolder nucleusHolder;

    private static final Color OVERLAY_COLOR = Color.red;

    public OverlayImageCanvas(ImagePlus imp, NucleusHolder nucleusHolder) {
        super(imp);
        this.nucleusHolder = nucleusHolder;
    }

    private static Point nucleusEntryTextPoint(NucleusEntry nucleusEntry){
        Roi roi = nucleusEntry.getRoi();
        return Point.builder().x((int) roi.getXBase()).y((int) roi.getYBase()).build();
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        applyNucleusEntriesToGraphics(nucleusHolder.getNucleusEntrys(), g);
    }


    private void applyNucleusEntriesToGraphics(Set<NucleusEntry> nucleusEntries, Graphics g) {
        for (NucleusEntry nucleusEntry : nucleusEntries){
            applyNucleusEntryToGraphics(nucleusEntry, g);
        }
    }

    private void applyNucleusEntryToGraphics(NucleusEntry nucleusEntry, Graphics g) {
        Roi roi = nucleusEntry.getRoi();
        if (roi == null){
            return;
        }
        Color oldColor = g.getColor();
        g.setColor(OVERLAY_COLOR);
        try {
            roi.draw(g);
            Point point = nucleusEntryTextPoint(nucleusEntry);
            g.drawString(nucleusEntry.getName(), screenX(point.getX()), screenY(point.getY()));
        } finally {
            g.setColor(oldColor);
        }
    }

}
