package edu.jhmi.telometer.util;

import edu.jhmi.telometer.bean.Cell;
import edu.jhmi.telometer.bean.NodeType;
import edu.jhmi.telometer.bean.Project;
import edu.jhmi.telometer.bean.Scoring;
import edu.jhmi.telometer.bean.Telomere;
import edu.jhmi.telometer.bean.TreeRoot;
import edu.jhmi.telometer.interfce.TreeKey;
import edu.jhmi.telometer.thirdparty.SpringUtilities;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import javax.swing.JLabel;
import javax.swing.JPanel;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Arrays;
import java.util.Map;

@Slf4j
public class TreeGuiUtil {




    public static<T extends TreeKey> void init(JPanel mainPanel, Map<T, SuperLabel> map, T[] keys) {
        initMap(map, keys);
        initMainPanel(mainPanel, map, keys);
    }

    public static<T extends TreeKey> void initMap(Map<T, SuperLabel> map, T[] keys) {
        for (T value : keys){
            map.put(value, superLabelForKey(value));
        }
    }

    private static <T extends TreeKey> SuperLabel superLabelForKey(T key) {
        JLabel keyLabel = new JLabel(StringUtils.capitalize(key.toString()));
        JLabel valueLabel = new JLabel("---------------------------------------------------------------------------------");
        return SuperLabel.builder()
                .label(keyLabel)
                .value(valueLabel)
                .build();
    }


    public static<T extends TreeKey> void initMainPanel(JPanel mainPanel, Map<T, SuperLabel> map, T[] keys) {
        for (TreeKey treeKey : keys){
            SuperLabel superLabel = map.get(treeKey);
            mainPanel.add(superLabel.getLabel());
            mainPanel.add(superLabel.getValue());
        }

        SpringUtilities.makeCompactGrid(mainPanel,
                map.keySet().size(), 2,
                6, 6,
                6, 6);
    }

    public static<T extends TreeKey> void setText(T treeKey, int newValue, Map<T, SuperLabel> map) {
        setText(treeKey, String.valueOf(newValue), map);
    }

    private static<T extends TreeKey> SuperLabel superLabelForTreeKey(T treeKey, Map<T, SuperLabel> map){
        SuperLabel superLabel = map.get(treeKey);
        Assert.notNull(superLabel, "no superLabel for treeKey: " + treeKey);
        return superLabel;
    }

    public static<T extends TreeKey> void setText(T treeKey, Object newValue, Map<T, SuperLabel> map) {
        SuperLabel superLabel = superLabelForTreeKey(treeKey, map);
        String stringValue = newValue != null ? String.valueOf(newValue) : "";
        superLabel.getValue().setText(stringValue);
    }

    public static<T extends TreeKey> void setKeyToolTip(T treeKey, String toolTip, Map<T, SuperLabel> map) {
        SuperLabel superLabel = superLabelForTreeKey(treeKey, map);
        superLabel.getLabel().setToolTipText(toolTip);
    }

    public static<T extends TreeKey> void setValueToolTip(T treeKey, String toolTip, Map<T, SuperLabel> map) {
        SuperLabel superLabel = superLabelForTreeKey(treeKey, map);
        superLabel.getValue().setToolTipText(toolTip);
    }

    public static<T extends TreeKey> void setBothToolTips(T treeKey, String toolTip, Map<T, SuperLabel> map) {
        setKeyToolTip(treeKey, toolTip, map);
        setValueToolTip(treeKey, toolTip, map);
    }

    public static<T extends TreeKey> void addMouseListener(T treeKey, MouseListener mouseListener, Map<T, SuperLabel> map){
        SuperLabel superLabel = superLabelForTreeKey(treeKey, map);
        JLabel label = superLabel.getValue();
        if (Arrays.asList(label.getMouseListeners()).contains(mouseListener)){
            return;
        }
        label.addMouseListener(mouseListener);
    }

    public static NodeType nodeTypeForClass(Class clazz){
        if (clazz.equals(TreeRoot.class)) {
            return NodeType.ROOT;
        }
        if (clazz.equals(Project.class)) {
            return NodeType.PROJECT;
        }
        if (clazz.equals(Scoring.class)) {
            return NodeType.SCORING;
        }
        if (clazz.equals(Cell.class)) {
            return NodeType.CELL;
        }
        if (clazz.equals(Telomere.class)) {
            return NodeType.TELOMERE;
        }
        throw new RuntimeException("Don't know what nodeType this class is " + clazz);
    }
}
