package edu.jhmi.telometer.view.about;

import edu.jhmi.telometer.interfce.HardConstants;
import edu.jhmi.telometer.thirdparty.SpringUtilities;
import edu.jhmi.telometer.util.EnvironmentUtil;
import edu.jhmi.telometer.util.FrameUtil;
import edu.jhmi.telometer.util.MemoryUtil;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class EnvironmentDialog {

    private final JButton closeButton = new JButton("Close");
    private final JPanel mainPanel = new JPanel(new BorderLayout());
    private JDialog dialog;

    public EnvironmentDialog() {
        JPanel dataPanel = new JPanel(new SpringLayout());
        MemoryUtil.MemoryStat memoryStat = MemoryUtil.getMemoryStat();
        EnvironmentUtil.EnvironmentStat environmentStat = EnvironmentUtil.getEnvironmentStat();

        final String[][] data = {
                {
                        "Used Memory",
                        memoryStat.usedMemoryString
                },
                {
                        "Total Memory",
                        memoryStat.totalMemoryString
                },
                {
                        "Max Memory",
                        memoryStat.maxMemoryString
                },
                {
                        "% Memory Used",
                        memoryStat.percentMemoryUsedString
                },
                {
                        "OS Name",
                        environmentStat.getOsName()
                },
                {
                        "OS Version",
                        environmentStat.getOsVersion()
                },
                {
                        "OS Arch",
                        environmentStat.getOsArch()
                },
                {
                        "Java Vendor",
                        environmentStat.getJavaVendor()
                },
                {
                        "Java Version",
                        environmentStat.getJavaVersion()
                },
                {
                        "Telometer Version",
                        HardConstants.VERSION
                },


        };

        final int rows = data.length;
        for (int i = 0; i < rows; i++) {
            String name = String.format("%s:", data[i][0]);
            String value = data[i][1];
            dataPanel.add(new JLabel(name));
            dataPanel.add(new JLabel(value));
        }

        SpringUtilities.makeCompactGrid(dataPanel, rows, 2, 5, 5, 5, 5);

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(closeButton);

        mainPanel.add(dataPanel, BorderLayout.CENTER);
        mainPanel.add(buttonPanel, BorderLayout.SOUTH);

        closeButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                dialog.dispose();
            }
        });
    }

    public void displayDialog(JDialog parentDialog) {
        dialog = new JDialog(parentDialog, "Environment", true);
        dialog.getContentPane().add(mainPanel);
        dialog.pack();
        FrameUtil.setRelativeLocation(dialog, parentDialog);
        dialog.setVisible(true);
    }
}
