package edu.jhmi.telometer.view.score;

import edu.jhmi.telometer.bean.Preset;
import edu.jhmi.telometer.bean.Project;
import edu.jhmi.telometer.thirdparty.SpringUtilities;
import edu.jhmi.telometer.util.ComboBoxUtil;
import org.springframework.util.Assert;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import java.util.List;


public class OptionsPanel {
    private final String[] DEFAULT_ITEMS = {"----NOT INITIALIZED----"};
    private final JComboBox projectComboBox = new JComboBox(DEFAULT_ITEMS);
    private final JTextField tagTextField = new JTextField();
    private final JComboBox presetComboBox = new JComboBox(DEFAULT_ITEMS);
    private final JCheckBox subtractCy3BackgroundCheckbox = new JCheckBox("Subtract CY3 Background", true);
    private final JPanel mainPanel = initPanel();

    public OptionsPanel(){

    }


    private JPanel initPanel() {
        setToolTips();
        JPanel formPanel = new JPanel();

        JLabel[] labels = {
                new JLabel("Project"),
                new JLabel("Scoring Tag"),
                new JLabel("Preset"),
                new JLabel("Option"),
        };

        JComponent[] components = {
                projectComboBox,
                tagTextField,
                presetComboBox,
                subtractCy3BackgroundCheckbox,
        };
        for (int i = 0; i < labels.length; i++) {
            formPanel.add(labels[i]);
            formPanel.add(components[i]);
            labels[i].setToolTipText(components[i].getToolTipText());
        }
        formPanel.setLayout(new SpringLayout());
        final int buffer = 100;
        SpringUtilities.makeCompactGrid(formPanel,
                labels.length, 2,
                2, 40,
                40, 40);
        return formPanel;
    }

    private static final String subtractCy3BackgroundTt = "Subtracts the CY3 background noise before doing calculations. \n"
            +"The background noise is the average intensity of the pixels in the CY3 image, that correspond to the circled cell in the DAPI image.";


    private void setToolTips() {

        projectComboBox.setToolTipText("Controls which project this scoring will go into");
        tagTextField.setToolTipText("assign a unique name to identify this scoring.  This may be a outside source id, or anything you choose");
        presetComboBox.setToolTipText("Controls options such as minimum and maximum spotSizes.  See Admin area add a new preset");
        subtractCy3BackgroundCheckbox.setToolTipText(subtractCy3BackgroundTt);

    }

    public JPanel getPanel() {
        return mainPanel;
    }

    public void setPresets(List<Preset> presets) {
        Assert.notEmpty(presets, "there must be at least one preset");
        presetComboBox.setModel(ComboBoxUtil.listToComboBoxModel(presets));
    }

    public void setProjects(List<Project> projects){
        Assert.notEmpty(projects, "there must be at least one project");
        projectComboBox.setModel(ComboBoxUtil.listToComboBoxModel(projects));
    }


    public Project getProject(){
        return (Project) projectComboBox.getSelectedItem();
    }

    public Preset getPreset(){
        return (Preset) presetComboBox.getSelectedItem();
    }

    public String getTag(){ return tagTextField.getText();}

    public boolean getSubtractCy3(){
        return subtractCy3BackgroundCheckbox.isSelected();
    }

    public void setTag(String tag) {
        tagTextField.setText(tag);
    }
}
