package edu.jhmi.telometer.view.score;


import edu.jhmi.telometer.interfce.ScorePanelListener;
import edu.jhmi.telometer.bean.CardType;
import edu.jhmi.telometer.util.FrameUtil;
import edu.jhmi.telometer.util.IconUtil;
import lombok.extern.slf4j.Slf4j;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;

@Slf4j
public class ScorePanel {

    private static final String NEXT = "Next";

    private final JPanel mainPanel = new JPanel();
    private final JFrame mainFrame = new JFrame();
    private final JButton resetButton = new JButton("Reset", IconUtil.getIcon("reset"));
    private final JButton nextButton = new JButton(NEXT, IconUtil.getIcon("arrowForward"));
    private final JTextArea instructionsTextArea = new JTextArea();
    private final JScrollPane instructionsScrollPane = new JScrollPane(instructionsTextArea);
    private final OptionsPanel optionsPanel = new OptionsPanel();
    private final ThresholdPanel thresholderPanel = new ThresholdPanel();
    private final NucleiiPanel nucleiiPanel = new NucleiiPanel();
    private final CompletePanel completePanel = new CompletePanel();
    private final ToolbarPanel toolbarPanel = new ToolbarPanel();
    private final CardLayout cardLayout = new CardLayout();
    private final JPanel cardPanel = new JPanel(cardLayout);
    private final JPanel nonePanel = new JPanel();
    private ScorePanelListener scorePanelListener;
    private boolean isInitted;

    public ScorePanel(){
        initGuiIfNecessary();
    }

    private void initGuiIfNecessary() {
        if (isInitted){
            return;
        }
        FrameUtil.setInitialLocation(mainFrame);
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(nextButton);
        buttonPanel.add(resetButton);
        JPanel northPanel = new JPanel(new BorderLayout());
        northPanel.add(buttonPanel, BorderLayout.WEST);
        nextButton.addActionListener(a -> scorePanelListener.nextButtonClicked());
        resetButton.addActionListener(a -> scorePanelListener.resetButtonClicked());

        instructionsTextArea.setEditable(false);
        instructionsTextArea.setRows(4);
        instructionsTextArea.setBackground(Color.LIGHT_GRAY);
        final String spaces = "------------------------------------------------------------------------------------------";

        instructionsTextArea.setText(String.format("%s\n%s\n%s\n%s\n%s", spaces, spaces, spaces, spaces, spaces));


        cardPanel.add(CardType.NONE.toString(), nonePanel);
        cardPanel.add(CardType.GATHER_INFO.toString(), optionsPanel.getPanel());
        cardPanel.add(CardType.THRESHOLDER.toString(), thresholderPanel.getPanel());
        cardPanel.add(CardType.ADD_NUCLEII.toString(), nucleiiPanel.getPanel());
        cardPanel.add(CardType.REMOVE_HALOS.toString(), toolbarPanel.getPanel());
        cardPanel.add(CardType.COMPLETE.toString(), completePanel.getPanel());

        thresholderPanel.setWidth((int) (instructionsTextArea.getPreferredSize().getWidth() * .80));

        mainPanel.setLayout(new BorderLayout());

        JPanel centerPanel = new JPanel(new BorderLayout());
        centerPanel.add(instructionsScrollPane, BorderLayout.NORTH);
        centerPanel.add(cardPanel, BorderLayout.SOUTH);

        mainPanel.add(northPanel, BorderLayout.NORTH);
        mainPanel.add(centerPanel, BorderLayout.CENTER);

        mainFrame.getContentPane().add(mainPanel);
        mainFrame.pack();
        mainFrame.setTitle("Score");
        isInitted = true;
        setToolTips();
    }

    private void setToolTips() {
        nextButton.setToolTipText("Next step");
        resetButton.setToolTipText("Score another image pair (If you haven't reached the end by clicking the complete button, " +
                "no scores will be saved)");
    }


    public void setInstructionsText(String text){
        log.info("User instructed: '{}'", text);
        instructionsTextArea.setText(text);
    }

    public void setScorePanelVisible(boolean isVisible) {
        mainFrame.setVisible(isVisible);
    }

    public void setResetButtonEnabled(boolean isEnabled) {
        resetButton.setEnabled(isEnabled);
    }

    public void setNextButtonStatus(NextButtonStatus nextButtonStatus){
        boolean isEnabled = nextButtonStatus == NextButtonStatus.NEXT_ENABLED
                || nextButtonStatus == NextButtonStatus.COMPLETE_ENABLED;
        String text = nextButtonStatus == NextButtonStatus.COMPLETE_ENABLED ? "Complete" : "Next";
        boolean isVisible = nextButtonStatus != NextButtonStatus.HIDDEN;
        nextButton.setEnabled(isEnabled);
        nextButton.setVisible(isVisible);
        nextButton.setText(text);
    }


    public void setNextButtonComplete(boolean isComplete){ nextButton.setText(isComplete ? "Finish" : NEXT);}

    public void setScorePanelListener(ScorePanelListener scorePanelListener){
        this.scorePanelListener = scorePanelListener;
        nucleiiPanel.setScorePanelListener(scorePanelListener);
        toolbarPanel.setScorePanelListener(scorePanelListener);
        thresholderPanel.setScorePanelListener(scorePanelListener);
        completePanel.setScorePanelListener(scorePanelListener);
    }

    public Component getParentComponent(){
        return mainFrame;
    }

    public void setCardPanelView(CardType cardType){
        cardLayout.show(cardPanel, cardType.toString());
    }

    public ThresholdPanel getThresholderPanel(){
        return thresholderPanel;
    }

    public NucleiiPanel getNucleiiPanel() {
        return nucleiiPanel;
    }

    public CompletePanel getCompletePanel() {
        return completePanel;
    }

    public ToolbarPanel getToolbarPanel() {
        return toolbarPanel;
    }

    public OptionsPanel getOptionsPanel() {
        return optionsPanel;

    }

    public enum NextButtonStatus{
        NEXT_ENABLED,
        NEXT_DISABLED,
        HIDDEN,
        COMPLETE_ENABLED
    }
}
