package edu.jhmi.telometer.view.start;

import edu.jhmi.telometer.bean.Scoring;
import edu.jhmi.telometer.controller.ScoreController;
import edu.jhmi.telometer.interfce.ResultListener;
import edu.jhmi.telometer.service.api.AdminService;
import edu.jhmi.telometer.service.api.TableService;
import edu.jhmi.telometer.service.api.TreeService;
import edu.jhmi.telometer.thirdparty.SpringUtilities;
import edu.jhmi.telometer.util.FrameUtil;
import edu.jhmi.telometer.util.IconUtil;
import edu.jhmi.telometer.util.api.ImageDirUtil;
import edu.jhmi.telometer.view.about.AboutDialog;
import edu.jhmi.telometer.view.admin.AdminPanel;
import edu.jhmi.telometer.view.query.table.TablesPanel;
import edu.jhmi.telometer.view.query.tree.TreePanel;
import lombok.extern.slf4j.Slf4j;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SpringLayout;
import java.awt.event.ActionListener;
import java.util.Arrays;

//FIXME tools tips on refresh button
//FIXME tool tips on admin app
//FIXME add maven site page -- update documentation!! --we copied it over
//FIXME integration test

@Slf4j
public class GeneralPanel {

    private final JButton scoreButton = new JButton("Score", icon("score"));
    private final JButton exportButton = new JButton("Export", icon("table"));
    private final JButton treeButton = new JButton("View", icon("tree"));
    private final JButton adminButton = new JButton("Admin", icon("settings"));
    private final JButton aboutButton = new JButton("About", icon("info"));
    private final JButton exitButton = new JButton("Exit", icon("exit"));

    private final JPanel mainPanel = new JPanel(new SpringLayout());
    private final JFrame frame = new JFrame("Telometer");

    private final AboutDialog aboutDialog = new AboutDialog();
    private final AdminPanel adminPanel = new AdminPanel();
    private final TreePanel treePanel = new TreePanel();
    private final TablesPanel tablesPanel = new TablesPanel();

    private ResultListener resultListener = new ResultListener() {
        @Override
        public void scoringSelectedForTable(Scoring scoring) {
            tablesPanel.setScoring(scoring);
            tablesPanel.display();
        }

        @Override
        public void scoringSelectedForTree(Scoring scoring) {
            treePanel.getTreeOnlyPanel().setScoring(scoring);
            treePanel.display();
        }
    };

    private ScoreController scoreController;

    public GeneralPanel() {
        setToolTips();
        ActionListener actionListener = a -> buttonClicked(a.getSource());
        JButton[] buttons = {scoreButton, exportButton, treeButton, adminButton, aboutButton, exitButton};
        Arrays.stream(buttons).forEach(b -> mainPanel.add(b));
        Arrays.stream(buttons).forEach(b -> b.addActionListener(actionListener));
        SpringUtilities.makeGrid(mainPanel, 6, 1, 5, 5, 5, 5);

        frame.getContentPane().add(mainPanel);
        frame.pack();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    }

    private void setToolTips() {
        scoreButton.setToolTipText("Start scoring!");
        exportButton.setToolTipText("Export results from a scoring to a text field / view the results in tables");
        treeButton.setToolTipText("View Results from scoring in a tree-like format / Also view images in scoring");
        adminButton.setToolTipText("Add/Edit/Remove values from Presets, Projects, and TissueTypes/LesionTypes/CellTypes");
        aboutButton.setToolTipText("Displays the version number, data directory, and license info");
        exitButton.setToolTipText("Close down the app");
    }

    private void buttonClicked(Object source) {
        if (source == aboutButton) {
            aboutDialog.showDialog(frame);
        } else if (source == treeButton) {
            treePanel.display();
        } else if (source == exportButton) {
            tablesPanel.display();
        } else if (source == scoreButton) {
            scoreController.begin();
        } else if (source == adminButton) {
            adminPanel.display();
        } else if (source == exitButton) {
            exit();
        } else {
            throw new RuntimeException("bad source: " + source);
        }
    }

    private void exit() {
        log.info("exiting...");
        System.exit(0);
    }


    public void display() {
        frame.setVisible(true);
    }

    private static ImageIcon icon(String name) {
        return IconUtil.getIcon(name);
    }

    public void setScoreController(ScoreController scoreController) {
        this.scoreController = scoreController;
        scoreController.setResultListener(resultListener);
    }

    public void setImageDirUtil(ImageDirUtil imageDirUtil) {
        treePanel.getNodeInfoPanel().setImageDirUtil(imageDirUtil);
        aboutDialog.setImageDirUtil(imageDirUtil);
    }

    public void setTreeService(TreeService treeService) {
        treePanel.getTreeOnlyPanel().setTreeService(treeService);
    }

    public void setTableService(TableService tableService) {
        tablesPanel.setTableService(tableService);
    }

    public void setAdminService(AdminService adminService) {
        adminPanel.setAdminService(adminService);
    }

}
